<?php 

add_action( 'wp_enqueue_scripts', 'salient_child_enqueue_styles', 100);

function salient_child_enqueue_styles() {
		
		$nectar_theme_version = nectar_get_theme_version();
		wp_enqueue_style( 'salient-child-style', get_stylesheet_directory_uri() . '/style.css', '', $nectar_theme_version );
		
    if ( is_rtl() ) {
   		wp_enqueue_style(  'salient-rtl',  get_template_directory_uri(). '/rtl.css', array(), '1', 'screen' );
		}
}

function custom_posts_list_shortcode($atts) {
    ob_start();

    // Set default attributes and override with user-defined attributes
    $atts = shortcode_atts(
        array(
            'posts_per_page' => 6,
        ),
        $atts,
        'custom_posts_list'
    );

    // WP Query arguments
    $args = array(
        'posts_per_page' => intval($atts['posts_per_page']),
        'post_status'    => 'publish',
    );

    // Custom query
    $query = new WP_Query($args);

    if ($query->have_posts()) {
        echo '<div class="news">';
        echo '<h2>News</h2>';
        echo '<ul>';
        while ($query->have_posts()) {
            $query->the_post();
            echo '<li>';
            echo '<h3 class="news-title"><a href="' . get_permalink() . '">' . get_the_title() . '</a></h3>';
            echo '<span class="date">' . get_the_date('F j, Y') . '</span>';
            echo '<span class="time">' . get_the_time('g:i a') . '<span class="dot"></span></span>';
            echo '<span class="author">' . get_the_author() . '</span>';
            echo '</li>';
        }
        echo '</ul>';
        echo '</div>';
    } else {
        echo 'No posts found.';
    }

    // Reset post data
    wp_reset_postdata();

    return ob_get_clean();
}

// Register shortcode
add_shortcode('custom_posts_list', 'custom_posts_list_shortcode');



// Hook into the 'init' action
add_action('init', 'create_event_post_type');

function create_event_post_type() {
    // Set up labels
    $labels = array(
        'name'                  => _x('Events', 'Post Type General Name', 'text_domain'),
        'singular_name'         => _x('Event', 'Post Type Singular Name', 'text_domain'),
        'menu_name'             => __('Events', 'text_domain'),
        'name_admin_bar'        => __('Event', 'text_domain'),
        'archives'              => __('Event Archives', 'text_domain'),
        'attributes'            => __('Event Attributes', 'text_domain'),
        'parent_item_colon'     => __('Parent Event:', 'text_domain'),
        'all_items'             => __('All Events', 'text_domain'),
        'add_new_item'          => __('Add New Event', 'text_domain'),
        'add_new'               => __('Add New', 'text_domain'),
        'new_item'              => __('New Event', 'text_domain'),
        'edit_item'             => __('Edit Event', 'text_domain'),
        'update_item'           => __('Update Event', 'text_domain'),
        'view_item'             => __('View Event', 'text_domain'),
        'view_items'            => __('View Events', 'text_domain'),
        'search_items'          => __('Search Event', 'text_domain'),
        'not_found'             => __('Not found', 'text_domain'),
        'not_found_in_trash'    => __('Not found in Trash', 'text_domain'),
        'featured_image'        => __('Featured Image', 'text_domain'),
        'set_featured_image'    => __('Set featured image', 'text_domain'),
        'remove_featured_image' => __('Remove featured image', 'text_domain'),
        'use_featured_image'    => __('Use as featured image', 'text_domain'),
        'insert_into_item'      => __('Insert into event', 'text_domain'),
        'uploaded_to_this_item' => __('Uploaded to this event', 'text_domain'),
        'items_list'            => __('Events list', 'text_domain'),
        'items_list_navigation' => __('Events list navigation', 'text_domain'),
        'filter_items_list'     => __('Filter events list', 'text_domain'),
    );

    // Set up arguments
    $args = array(
        'label'                 => __('Event', 'text_domain'),
        'description'           => __('Event Description', 'text_domain'),
        'labels'                => $labels,
        'supports'              => array('title', 'editor', 'excerpt', 'author', 'thumbnail', 'comments', 'revisions', 'custom-fields'),
        'hierarchical'          => false,
        'public'                => true,
        'show_ui'               => true,
        'show_in_menu'          => true,
        'menu_position'         => 5,
        'show_in_admin_bar'     => true,
        'show_in_nav_menus'     => true,
        'can_export'            => true,
        'has_archive'           => true,
        'exclude_from_search'   => false,
        'publicly_queryable'    => true,
        'capability_type'       => 'post',
        'show_in_rest'          => true, // For Gutenberg support
        'menu_icon'             => 'dashicons-calendar', // Dashicons for event
    );

    // Register the post type
    register_post_type('event', $args);
}


function custom_events_list_shortcode($atts) {
    ob_start();

    // Set default attributes and override with user-defined attributes
    $atts = shortcode_atts(
        array(
            'posts_per_page' => -1
        ),
        $atts,
        'custom_event_list'
    );

    // Get the current date
    $current_date = date('Ymd'); // Format the date in the same format as stored in the 'start_date' field

    // WP Query arguments
    $args = array(
        'post_type'      => 'event',
        'posts_per_page' => intval($atts['posts_per_page']),
        'post_status'    => 'publish',
        'meta_query'     => array(
            array(
                'key'     => 'start_date',
                'value'   => $current_date,
                'compare' => '>=',
                'type'    => 'DATE'
            )
        ),
        'meta_key'   => 'start_date', // Key for ordering
        'orderby'    => 'meta_value', // Order by start_date
        'order'      => 'ASC',        // Ascending order
    );

    // Custom query
    $query = new WP_Query($args);

    if ($query->have_posts()) {
        echo '<div class="events">';
        echo '<ul>';
        while ($query->have_posts()) {
            $query->the_post();
            $event_flag = get_field('flag');
            $event_date = get_field('start_date');
            $event_location = get_field('location');

            echo '<li>';
            echo '<h3 class="news-title"><a href="' . get_permalink() . '">' . get_the_title() . '</a></h3>';
            
            if ($event_date || $event_flag || $event_location) {
                echo '<div class="loc-flag">';
                
                if ($event_date) {
                    echo '<span class="date">Date: ' . esc_html($event_date) . '</span>';
                }
                
                if ($event_flag) {
                    echo '<div class="location"><img src="' . esc_url($event_flag) . '" alt="Flag">';
                }

                if ($event_location) {
                    echo '<span>' . esc_html($event_location) . '</span>';
                }

                if ($event_flag) {
                    echo '</div>'; // Closing location div if flag exists
                }
                
                echo '</div>'; // Closing loc-flag div
            }

            echo '</li>';
        }
        echo '</ul>';
        echo '</div>';
    } else {
        echo 'No upcoming events found.';
    }

    // Reset post data
    wp_reset_postdata();

    return ob_get_clean();
}

// Register shortcode
add_shortcode('custom_event_list', 'custom_events_list_shortcode');





function custom_posts_list_shortcode_meeting($atts) {
    ob_start();

    // Set default attributes and override with user-defined attributes
    $atts = shortcode_atts(
        array(
            'posts_per_page' => -1,
        ),
        $atts,
        'custom_posts_list_meeting'
    );

    // WP Query arguments
    $args = array(
        'posts_per_page' => intval($atts['posts_per_page']),
        'post_status'    => 'publish',
        'post_type'      => 'event', // Specify the custom post type
    );

    // Custom query
    $query = new WP_Query($args);

    if ($query->have_posts()) {
        echo '<div class="posts-list">';

        // Display the most recent post prominently
        $query->the_post();
        $event_date = get_field('start_date');
        $event_end_date = get_field('end_date');
        $event_location = get_field('location');
        
        echo '<div class="recent-post">';
        echo '<h2 class="recent-post-title"><a href="' . get_permalink() . '">' . get_the_title() . '</a></h2>';
        echo '<span class="date"><strong>Start From:</strong> ' . esc_html($event_date) . ' to ' . esc_html($event_end_date) . '</span><br>';
        echo '<span class="location"><strong>Location:</strong> ' . esc_html($event_location) . '</span><br>';
        echo '<div class="excerpt">' . get_the_excerpt() . '</div>';
        echo '</div>';

        // Display previous posts in a list
        if ($query->post_count > 1) {
            echo '<table class="previous-posts" style="border:0;">';
            while ($query->have_posts()) {
                $query->the_post();
                $event_date = get_field('start_date');
                echo '<tr class="previous-post" style="border:0;">';
                echo '<td class="previous-post-date" style="border:0;">' . esc_html($event_date) . '</td>';
                echo '<td class="previous-post-title" style="border:0;"><a href="' . get_permalink() . '">' . get_the_title() . '</a></td>';
                echo '</td>';
            }
            echo '</table>';
        }

        echo '</div>';
    } else {
        echo 'No posts found.';
    }

    // Reset post data
    wp_reset_postdata();

    return ob_get_clean();
}

// Register shortcode
add_shortcode('custom_posts_list_meeting', 'custom_posts_list_shortcode_meeting');




?>